<?php

namespace App\Http\Controllers\Sale;

use App\Http\Controllers\Controller;
use App\Models\Cart;
use App\Models\Customer;
use App\Models\Product;
use App\Models\ProductInventory;
use App\Models\SalesCart;
use App\Models\SalesCartItem;
use Exception;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Session;

class CartController extends Controller
{
    public function model()
    {
        return new SalesCart();
    }
    public function index(Request $request)
    {
        $customerID = session('selectedCustomerID');

        $cart = SalesCart::with('salescartitems.saleproduct')
            ->where('customer_id', $customerID)
            ->first();
        return view('sales.pages.carts', compact('cart'));
    }

    public function addToCart(Request $request)
    {
      
        $customerId = $request->customer_id;
        $customer = Customer::find($customerId);
        $products = $request->products;

        $cart = SalesCart::firstOrCreate(
            ['customer_id' => $customerId],
            [
                'name' => $customer->contact_person ?? null,
                'email' => $customer->email ?? null,
            ]
        );

        $itemsCount = $cart->salescartitems->count();
        $itemsQty = $cart->salescartitems->sum('quantity');
        $totalAmount = $cart->salescartitems->sum('total_amount');
        $totalTaxAmount = 0;

        foreach ($products as $productData) {
            $product = Product::with('tax')->find($productData['id']);
            $quantity = $productData['qty'];
            $price =$productData['price'] ;
            $taxRate = $product->tax->percent ?? 0;

            $taxAmount = ($price * $taxRate) / 100;
            $total = ($price + $taxAmount) * $quantity;

            $cartItem = SalesCartItem::where('cart_id', $cart->id)
                ->where('product_id', $product->id)
                ->first();

            if ($cartItem) {
                // Update the existing item
                $itemsQty -= $cartItem->quantity;
                $totalAmount -= $cartItem->total_amount;
                $totalTaxAmount -= $cartItem->tax_amount;

                $cartItem->quantity += $quantity;
                $cartItem->price = $price;
                $cartItem->tax_amount = $taxAmount * $quantity;
                $cartItem->total_amount = ($cartItem->price + $taxAmount) * $cartItem->quantity;
                $cartItem->save();

                $itemsQty += $cartItem->quantity;
                $totalAmount += $cartItem->total_amount;
                $totalTaxAmount += $cartItem->tax_amount;
            } else {
                // Add a new item
                SalesCartItem::create([
                    'cart_id' => $cart->id,
                    'product_id' => $product->id,
                    'quantity' => $quantity,
                    'price' => $price,
                    'tax_amount' => $taxAmount * $quantity,
                    'total_amount' => $total,
                    'name' => $product->name,
                    'sku' => $product->sku,
                ]);

                $itemsCount++;
                $itemsQty += $quantity;
                $totalAmount += $total;
                $totalTaxAmount += $taxAmount * $quantity;
            }
        }

        // Update cart totals
        $cart->update([
            'items_count' => $itemsCount,
            'items_qty' => $itemsQty,
            'total_amount' => $totalAmount,
            'tax' => $totalTaxAmount,
            'grand_total' => $totalAmount + $totalTaxAmount,
        ]);

        return response()->json([
            'success' => true,
            'message' => 'Products added to cart successfully!'
        ]);
    }


    public function getCartCount()
    {
        $customerid = session('selectedCustomerID');
        $cart = SalesCart::where('customer_id', $customerid)->first();
        if ($cart) {
            $itemscount = $cart->items_count;
        } else {
            $itemscount = 0;
        }
        return response()->json(['count' => $itemscount]);
    }


    public function deletecart($itemId)
    {
        $cartItem = SalesCartItem::find($itemId);

        if (!$cartItem) {
            return redirect()->back()->with('error', 'Cart item not found.');
        }
        $cart = $cartItem->salescart;
        $cartItem->delete();
        $grandTotal = 0;
        $hstAmount = 0;

        foreach ($cart->salescartitems as $item) {
            $grandTotal += $item->total_amount;
            $hstAmount += $item->tax_amount;
        }
        $totalAmount = $grandTotal + $hstAmount;
        $cart->update([
            'items_count' => $cart->salescartitems->count(),
            'items_qty' => $cart->salescartitems->sum('quantity'),
            'total_amount' => $grandTotal,
            'tax' => $hstAmount,
            'grand_total' => $totalAmount,
        ]);
        return redirect()->back()->with('message', 'Cart item deleted successfully.');
    }



    public function updateCartItem(Request $request)
    {
        $request->validate([
            'cart_id' => 'required|exists:sales_cart_items,id',
            'product_qty' => 'required|integer|min:1'
        ]);

        $cartItem = SalesCartItem::find($request->cart_id);

        if (!$cartItem) {
            return response()->json(['error' => 'Cart item not found'], 404);
        }

        $cartItem->quantity = $request->product_qty;

        $productPrice = $cartItem->saleproduct->price;
        $taxRate = $cartItem->saleproduct->tax->percent ?? 0;

        $itemTaxAmount = ($productPrice * $taxRate / 100) * $cartItem->quantity;
        $cartItem->tax_amount = $itemTaxAmount;

        $cartItem->total_amount = ($productPrice * $cartItem->quantity) + $itemTaxAmount;
        $cartItem->save();

        $cart = $cartItem->salescart;
        $cartItems = $cart->salescartitems;

        $grandTotal = 0;
        $totalTaxAmount = 0;

        foreach ($cartItems as $item) {
            $grandTotal += $item->total_amount;
            $totalTaxAmount += $item->tax_amount;
        }

        $cart->tax = $totalTaxAmount;
        $cart->grand_total = $grandTotal;
        $cart->total_amount = $grandTotal - $totalTaxAmount;
        $cart->save();

        return response()->json([
            'success' => true,
            'cartData' => [
                'id' => $cartItem->id,
                'quantity' => $cartItem->quantity,
                'total_selling_price' => number_format($cartItem->total_amount, 2)
            ],
            'grand_amount' => number_format($cart->grand_total, 2),
            'hst_amount' => number_format($cart->tax, 2),
            'total_amount' => number_format($cart->total_amount, 2)
        ]);
    }
}
